using System;
using System.Xml;
using System.Reflection;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace MySerialization
{
  public class NullValue
  {
    public NullValue()
    {
    }
  }

  public class ObjectSerializer
  {
    public static object Deserialize(XmlDocument doc)
    {
      return Deserialize(doc.InnerXml);
    }

    public static object Deserialize(string input)
    {
      //We put the input string back into an xml document instance. This will make life
      //easier in the recursion, because we can loop through the xml node objects.
      XmlDocument doc = new XmlDocument();
      doc.LoadXml(input);

      //Retrieve the type of the serialized object, using the Type-Attribute we added 
      //during the serialization. Using this type information, we can create an instance
      //of this type.
      Type mainType = Type.GetType(doc.DocumentElement.Attributes["Type"].Value);
      object result = Activator.CreateInstance(mainType);

      //Any NullValue instances we created during serialization to replace null values will
      //be transformed back into null values.
      if (result is NullValue)
        result = null;

      if (result != null)
      {
        //Now we get all properties of our object type so we can start recursion.
        PropertyInfo[] properties = mainType.GetProperties();
        foreach (PropertyInfo pi in properties)
        {
          //For this article we exclude indexed properties like type accessors.
          ParameterInfo[] paramInfos = pi.GetIndexParameters();
          if (paramInfos.Length == 0)
            DeserializeRecursive(pi, result, doc.DocumentElement);
        }
      }

      return result;
    }

    private static void DeserializeRecursive(PropertyInfo pi, object parent, XmlNode parentNode)
    {
      //Loop through all child nodes of the current parent node to find the node which
      //belongs to the current property.
      foreach (XmlNode node in parentNode.ChildNodes)
      {
        if (node.Name == pi.Name)
        {
          //Case 1 : current property is primitive, enum type ot string
          if (pi.PropertyType.IsPrimitive || pi.PropertyType == typeof(string))
            //Use Convert.ChangeType as a universal casting method for primitives
            pi.SetValue(parent, Convert.ChangeType(node.InnerText, Type.GetType(node.Attributes["Type"].Value)), null);
          else if (pi.PropertyType.IsEnum)
            //Transform the value string back into the enum value.
            pi.SetValue(parent, Enum.Parse(Type.GetType(node.Attributes["Type"].Value), node.InnerText), null);
          //Case 2 : current property is a list type
          else if (typeof(IList).IsAssignableFrom(pi.PropertyType))
          {
            //Create a new list we will fill and attach to the parent object as new property
            //value.
            IList temp = (IList)(Activator.CreateInstance(Type.GetType(node.Attributes["Type"].Value)));

            //Loop through all ListEntry child nodes to create the items in the list.
            foreach (XmlNode child in node.ChildNodes)
            {
              //Get the value node of this list entry
              XmlNode valueNode = child.ChildNodes[0];

              //Retrieve the type of the list entry item
              Type itemType = Type.GetType(valueNode.Attributes["Type"].Value);

              //Now handle the differenct cases for the list item and add the created item
              //to our working list.
              if (itemType.IsPrimitive || itemType == typeof(string))
                temp.Add(Convert.ChangeType(valueNode.InnerText, itemType));
              else if (itemType.IsEnum)
                temp.Add(Enum.Parse(itemType, valueNode.InnerText));
              else if (itemType == typeof(NullValue))
                temp.Add(null);
              else
              {
                object item = Activator.CreateInstance(itemType);
                PropertyInfo[] itemProps = itemType.GetProperties();
                foreach (PropertyInfo itemProp in itemProps)
                {
                  ParameterInfo[] paramInfos = itemProp.GetIndexParameters();
                  if (paramInfos.Length == 0)
                    DeserializeRecursive(itemProp, item, valueNode);
                }

                temp.Add(item);
              }
            }

            //Finally set our working list as new value for the current property from the
            //parent object.
            pi.SetValue(parent, temp, null);
          }
          //Case 3 : Current property is a dictionary type
          else if (typeof(IDictionary).IsAssignableFrom(pi.PropertyType))
          {
            //This case is very similiar to the list case described above.
            //I won't explain it any further as it should be really self-explainatory now.
            IDictionary temp = (IDictionary)(Activator.CreateInstance(Type.GetType(node.Attributes["Type"].Value)));
            foreach (XmlNode child in node.ChildNodes)
            {
              XmlNode valueNode = child.ChildNodes[1];
              object value = null;

              Type valueType = Type.GetType(valueNode.Attributes["Type"].Value);
              if (valueType.IsPrimitive || valueType == typeof(string))
                value = Convert.ChangeType(valueNode.InnerText, valueType);
              else if (valueType.IsEnum)
                value = Enum.Parse(valueType, valueNode.InnerText);
              else if (valueType == typeof(NullValue))
                value = null;
              else
              {
                value = Activator.CreateInstance(valueType);
                PropertyInfo[] valueProps = valueType.GetProperties();
                foreach (PropertyInfo valProp in valueProps)
                {
                  ParameterInfo[] paramInfos = valProp.GetIndexParameters();
                  if (paramInfos.Length == 0)
                    DeserializeRecursive(valProp, value, valueNode);
                }
              }

              XmlNode keyNode = child.ChildNodes[0];
              object key = null;

              Type keyType = Type.GetType(keyNode.Attributes["Type"].Value);
              if (keyType.IsPrimitive || keyType == typeof(string))
                key = Convert.ChangeType(keyNode.InnerText, keyType);
              else if (keyType.IsEnum)
                key = Enum.Parse(keyType, keyNode.InnerText);
              else
              {
                key = Activator.CreateInstance(keyType);
                PropertyInfo[] keyProps = keyType.GetProperties();
                foreach (PropertyInfo keyProp in keyProps)
                {
                  ParameterInfo[] paramInfos = keyProp.GetIndexParameters();
                  if (paramInfos.Length == 0)
                    DeserializeRecursive(keyProp, key, keyNode);
                }
              }

              temp.Add(key, value);
            }

            pi.SetValue(parent, temp, null);
          }
          //If we found a NullValue instance, replace it by setting null as new value in the
          //current property of our parent object.
          else if (pi.PropertyType == typeof(NullValue))
          {
            pi.SetValue(parent, null, null);
          }
          //Case 4 : current property is a complex class/struct type
          else
          {
            //Create a new instance of this property type
            Type subType = Type.GetType(node.Attributes["Type"].Value);
            object subObj = Activator.CreateInstance(subType);

            //Now set the new created instance as value for the parent object's property.            
            pi.SetValue(parent, subObj, null);

            //Finally, recursively loop through the properties of this new instance like we did
            //in Deserialize() method.
            PropertyInfo[] subPIs = subType.GetProperties();
            foreach (PropertyInfo spi in subPIs)
            {
              ParameterInfo[] paramInfos = spi.GetIndexParameters();
              if (paramInfos.Length == 0)
                DeserializeRecursive(spi, subObj, node);
            }
          }
        }
      }
    }

    public static string Serialize(object o, out XmlDocument document)
    {
      XmlDocument doc = new XmlDocument();
      string s = Serialize(o);
      doc.LoadXml(s);
      document = doc;
      return s;
    }

    public static string Serialize(object input)
    {
      //Create the xml document instance we are going to work with as well as
      //a main element node
      XmlDocument doc = new XmlDocument();
      XmlNode mainElement = doc.CreateNode(XmlNodeType.Element, "Instance", "");

      if (input != null)
      {
        //We need to store the type information of the input object, because we
        //will need it to deserialize this again later. The type information is
        //stored in an attribute of the main element node.
        XmlAttribute att = doc.CreateAttribute("Type");
        att.Value = input.GetType().AssemblyQualifiedName;
        mainElement.Attributes.Append(att);

        //Add the main element node to the xml document.
        doc.AppendChild(mainElement);

        //Here we enter the recursion, passing the xml document instance, the parent
        //xml node of the next recursion level and the parent object of the next 
        //recursion level in as parameters. Because the xml document instance and the 
        //node element are reference types (classes) we don't need a "ref" here.
        //The object could be a value type like a primitive or an enum, but as we are
        //serializing, we will only need read access to the object, so no "ref" needed
        //here either.
        SerializeRecursive(doc, mainElement, input);

        //At the end of the recursion, we return the string representation of our xml
        //document instance.
        return doc.InnerXml;
      }
      else
      {
        //During deserialization, we would get in trouble if we were trying to retrieve
        //type information of a null value. Therefore we replace any null values with a
        //dummy instance of a type NullValue.
        return Serialize(new NullValue());
      }
    }

    //Keep in mind that the xml node is the parent node and the object is the parent object of the 
    //subobject we are going to serialize in the current recursion level.
    private static void SerializeRecursive(XmlDocument doc, XmlNode parentNode, object parentObject)
    {
      //Handle null value by using a NullValue instance as replacement.
      object o = new NullValue();
      if (parentObject != null)
        o = parentObject;

      Type objType = o.GetType();

      //Case 1: Parent object is a primitive or an enum (or a string)
      if (objType.IsPrimitive || objType.IsEnum)
      {
        //If our parent object is a primitive or an enum type, we have reached a leaf in the recursion.
        //A leaf always ends in a concrete value, which we then store as value of the last parent xml
        //node.
        parentNode.InnerText = o.ToString();
      }
      else if (objType == typeof(string))
      {
        //The string type has to be handled seperately, because it doesn't count as a primitive.
        parentNode.InnerText = (string)o;
      }
      //Case 2: Parent object is a list type
      else if (typeof(IList).IsAssignableFrom(objType))
      {
        //Unbox the list and loop through all elements, entering the next recursion level.       
        IList tempList = (IList)o;
        foreach (object i in tempList)
        {
          //For each entry in the list, we create a child node "ListEntry" and add it below
          //out current parent node.
          XmlNode node = doc.CreateNode(XmlNodeType.Element, "ListEntry", "");
          parentNode.AppendChild(node);

          //Below the ListEntry node, we add another child node which contains the value of
          //the list item.
          XmlNode valueNode = doc.CreateNode(XmlNodeType.Element, "Value", "");
          node.AppendChild(valueNode);

          //Handle the case when the list item is a null value, by replacing it with a NullValue
          //instance.
          object item = new NullValue();
          if (i != null)
            item = i;

          //Save the type information of the list item object for deserialization.
          //We add this type info as an attribute, in the same way we did with the main document
          //node.
          XmlAttribute att = doc.CreateAttribute("Type");
          att.Value = item.GetType().AssemblyQualifiedName;
          valueNode.Attributes.Append(att);

          //Enter the next recursion step, using the valueNode as new parent node and the list item
          //as new parent object.
          SerializeRecursive(doc, valueNode, item);
        }

      }
      //Case 3: Parent object is a dictionary type.
      else if (typeof(IDictionary).IsAssignableFrom(objType))
      {
        //This case works in about the same way as the list type serialization above
        //and should be quite self-explainatory if you understood the list case.
        //The only difference here is that the recursion tree will split into two
        //branches here, one for the key and one for the value of each dictionary
        //entry.
        IDictionary tempDictionary = (IDictionary)o;
        foreach (object key in tempDictionary.Keys)
        {
          XmlNode node = doc.CreateNode(XmlNodeType.Element, "DictionaryEntry", "");
          parentNode.AppendChild(node);

          XmlNode keyNode = doc.CreateNode(XmlNodeType.Element, "Key", "");
          XmlAttribute kAtt = doc.CreateAttribute("Type");
          kAtt.Value = key.GetType().AssemblyQualifiedName;
          keyNode.Attributes.Append(kAtt);

          node.AppendChild(keyNode);
          SerializeRecursive(doc, keyNode, key);

          XmlNode valueNode = doc.CreateNode(XmlNodeType.Element, "Value", "");
          XmlAttribute vAtt = doc.CreateAttribute("Type");

          object entry = new NullValue();
          if (tempDictionary[key] != null)
            entry = tempDictionary[key];

          vAtt.Value = entry.GetType().AssemblyQualifiedName;
          valueNode.Attributes.Append(vAtt);

          node.AppendChild(valueNode);
          SerializeRecursive(doc, valueNode, entry);
        }
      }
      //Case 4: Parent object is a complex type (class or struct)
      else
      {
        //This case looks similiar to what we did in the original Serialize() method.        
        //First we get all properties of the current object, then we loop through them,
        //continuing the recursion.
        PropertyInfo[] properties = objType.GetProperties();
        foreach (PropertyInfo pi in properties)
        {
          //Exclude indexed properties
          ParameterInfo[] test = pi.GetIndexParameters();
          if (test.Length == 0)
          {
            XmlNode node = doc.CreateNode(XmlNodeType.Element, pi.Name, "");
            XmlAttribute att = doc.CreateAttribute("Type");
            node.Attributes.Append(att);
            att.Value = pi.PropertyType.AssemblyQualifiedName;
            parentNode.AppendChild(node);

            //Get the concrete value of the current property in the parent object
            //so we can continue recursion based on that value object.
            SerializeRecursive(doc, node, pi.GetValue(o, null));
          }
        }
      }
    }
  }
}

